use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006008
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
Torsten
Raudssus
torsten
https
Social
Software
Karen
Etheridge
ether
Randy
Stauner
rwstauner
getty
Nickolay
Platonov
nplatonov
Olivier
Mengué
dolmen
Al
Newkirk
github
Tatsuhiko
Miyagawa
miyagawa
Thomas
Sibley
tsibley
lib
Dist
Zilla
Plugin
Run
AfterBuild
AfterMint
AfterRelease
BeforeBuild
BeforeRelease
Release
Role
Runner
Test
